<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

function eme_payment_form($payment_id) {

   $payment = eme_get_payment($payment_id);
   $payment_paid = eme_get_payment_paid($payment);
   if ($payment_paid) {
	   return "<div class='eme-already-paid'>".__('This has already been paid for','events-made-easy')."</div>";
   }
   if ($payment['member_id']) {
	   return eme_payment_member_form($payment_id);
   }

   $booking_ids = eme_get_payment_booking_ids($payment_id);
   if (!$booking_ids)
      return;
   if (count($booking_ids)==1) {
      $is_multi=0;
   } else {
      $is_multi=1;
   }

   $total_price=eme_get_payment_price($payment_id);

   // we take the currency of the first event in the series
   $booking=eme_get_booking($booking_ids[0]);
   $event=eme_get_event($booking['event_id']);
   $cur = $event['currency'];

   // if it is paid: show that and return
   if ($booking['waitinglist']) {
	   return "<div class='eme-waiting'>".__('This booking is on a waitinglist, payment is not possible.','events-made-easy')."</div>";
   }

   $ret_string = '';
   if (!$is_multi)
      $eme_payment_form_header_format=get_option('eme_payment_form_header_format');
   else
      $eme_payment_form_header_format=get_option('eme_multipayment_form_header_format');
   if (!empty($eme_payment_form_header_format)) {
      $result = eme_replace_booking_placeholders($eme_payment_form_header_format, $event, $booking, $is_multi);
      $ret_string .= "<div id='eme-payment-formtext' class='eme-payment-formtext'>";
      $ret_string .= $result;
      $ret_string .= "</div>";
   } else {
      $ret_string .= "<div id='eme-payment-handling' class='eme-payment-handling'>".__('Payment handling','events-made-easy')."</div>";
      $ret_string .= "<div id='eme-payment-price-info' class='eme-payment-price-info'>".sprintf(__("The price in %s is: %01.2f",'events-made-easy'),$cur,$total_price)."</div>";
   }
   $ret_string .= "<div id='eme-payment-form' class='eme-payment-form'>";
   $payment=eme_get_payment($payment_id);
   if ($event['use_paypal'])
      $ret_string .= eme_paypal_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['event_properties']['use_legacypaypal'])
      $ret_string .= eme_legacypaypal_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['use_2co'])
      $ret_string .= eme_2co_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['use_webmoney'])
      $ret_string .= eme_webmoney_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['use_fdgg'])
      $ret_string .= eme_fdgg_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['use_mollie'])
      $ret_string .= eme_mollie_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['use_sagepay'])
      $ret_string .= eme_sagepay_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['event_properties']['use_worldpay'])
      $ret_string .= eme_worldpay_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['event_properties']['use_stripe'])
      $ret_string .= eme_stripe_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['event_properties']['use_braintree'])
      $ret_string .= eme_braintree_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['event_properties']['use_paymill'])
      $ret_string .= eme_paymill_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['event_properties']['use_instamojo'])
      $ret_string .= eme_instamojo_form($event,$payment, $total_price,$cur,$booking['lang'],$is_multi);
   if ($event['event_properties']['use_offline']) {
      $eme_offline_format = get_option('eme_offline_payment');
      $result = eme_replace_booking_placeholders($eme_offline_format, $event, $booking, $is_multi);
      $ret_string .= "<div id='eme-payment-offline' class='eme-payment-offline'>";
      $ret_string .= $result;
      $ret_string .= "</div>";
   }
   $ret_string .= "</div>";

   if (!$is_multi)
      $eme_payment_form_footer_format=get_option('eme_payment_form_footer_format');
   else
      $eme_payment_form_footer_format=get_option('eme_multipayment_form_footer_format');
   if (!empty($eme_payment_form_footer_format)) {
      $result = eme_replace_booking_placeholders($eme_payment_form_footer_format, $event, $booking, $is_multi);
      $ret_string .= "<div id='eme-payment-formtext' class='eme-payment-formtext'>";
      $ret_string .= $result;
      $ret_string .= "</div>";
   }

   return $ret_string;
}

function eme_payment_member_form($payment_id) {

   $ret_string = "";

   $member = eme_get_member_by_paymentid($payment_id);
   $person = eme_get_person($member['person_id']);
   $total_price=eme_get_member_payment_price($payment_id);
   $membership=eme_get_membership($member['membership_id']);
   $cur = $membership['properties']['currency'];

   $eme_payment_form_header_format=eme_get_template_format($membership['properties']['payment_form_header_tpl']);
   if (!empty($eme_payment_form_header_format)) {
      $result = eme_replace_member_placeholders($eme_payment_form_header_format, $membership, $member);
      $ret_string .= "<div id='eme-payment-formtext' class='eme-payment-formtext'>";
      $ret_string .= $result;
      $ret_string .= "</div>";
   } else {
      $ret_string .= "<div id='eme-payment-handling' class='eme-payment-handling'>".__('Payment handling','events-made-easy')."</div>";
      $ret_string .= "<div id='eme-payment-price-info' class='eme-payment-price-info'>".sprintf(__("The price in %s is: %01.2f",'events-made-easy'),$cur,$total_price)."</div>";
   }
   $ret_string .= "<div id='eme-payment-form' class='eme-payment-form'>";
   $payment=eme_get_payment($payment_id);
   if ($membership['properties']['use_paypal'])
      $ret_string .= eme_paypal_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_legacypaypal'])
      $ret_string .= eme_legacypaypal_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_2co'])
      $ret_string .= eme_2co_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_webmoney'])
      $ret_string .= eme_webmoney_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_fdgg'])
      $ret_string .= eme_fdgg_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_mollie'])
      $ret_string .= eme_mollie_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_sagepay'])
      $ret_string .= eme_sagepay_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_worldpay'])
      $ret_string .= eme_worldpay_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_stripe'])
      $ret_string .= eme_stripe_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_braintree'])
      $ret_string .= eme_braintree_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_paymill'])
      $ret_string .= eme_paymill_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_instamojo'])
      $ret_string .= eme_instamojo_form($membership,$payment, $total_price,$cur,$person['lang']);
   if ($membership['properties']['use_offline']) {
      $eme_offline_format = eme_get_template_format($membership['properties']['offline_payment_tpl']);
      $result = eme_replace_member_placeholders($eme_offline_format, $membership, $member);
      $ret_string .= "<div id='eme-payment-offline' class='eme-payment-offline'>";
      $ret_string .= $result;
      $ret_string .= "</div>";
   }
   $ret_string .= "</div>";

   $eme_payment_form_footer_format=eme_get_template_format($membership['properties']['payment_form_footer_tpl']);
   if (!empty($eme_payment_form_footer_format)) {
      $result = eme_replace_member_placeholders($eme_payment_form_footer_format, $membership, $member);
      $ret_string .= "<div id='eme-payment-formtext' class='eme-payment-formtext'>";
      $ret_string .= $result;
      $ret_string .= "</div>";
   }
   return $ret_string;
}

function eme_payment_gateway_total($price,$cur,$gateway) {
   $price += eme_payment_gateway_extra_charge($price,$gateway);
   $eme_zero_decimal_currencies_arr=eme_zero_decimal_currencies();
   if (in_array($cur,$eme_zero_decimal_currencies_arr)) {
      $price = intval($price);
   } else {
      if ($gateway == "stripe") $price *=100;
      if ($gateway == "paymill") $price *=100;
   }
   return $price;
}

function eme_payment_gateway_extra_charge($price,$gateway) {
   $extra=get_option('eme_'.$gateway.'_cost');
   $result=0;
   if ($extra) {
	   if (strstr($extra,"%")) {
		   $extra=str_replace("%","",$extra);
		   $result += sprintf("%01.2f",$price*$extra/100);
	   } else {
		   $result += sprintf("%01.2f",$extra);
	   }
   }
   $extra=get_option('eme_'.$gateway.'_cost2');
   if ($extra) {
	   if (strstr($extra,"%")) {
		   $extra=str_replace("%","",$extra);
		   $result += sprintf("%01.2f",$price*$extra/100);
	   } else {
		   $result += sprintf("%01.2f",$extra);
	   }
   }
   return $result;
}

function eme_webmoney_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="webmoney";
   $eme_webmoney_purse=get_option('eme_webmoney_purse');
   if (!$eme_webmoney_purse) return;

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $success_link = eme_payment_return_url($payment,1);
   $fail_link = eme_payment_return_url($payment,2);
   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   $description=eme_esc_html($description);

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$line['currency'],$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$line['currency'],$lang);
   if (empty($button_label)) $button_label=$gateway;
   // webmoney api does this itself
   // $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$line['currency'],$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');

   require_once('payment_gateways/webmoney/webmoney.inc.php');
   $wm_request = new WM_Request();
   $wm_request->payment_amount =$price;
   $wm_request->payment_desc = $description;
   $wm_request->payment_no = $payment_id;
   $wm_request->payee_purse = $eme_webmoney_purse;
   $wm_request->success_method = WM_POST;
   $result_link = add_query_arg(array('eme_eventAction'=>'webmoney'),$events_page_link);

   $wm_request->result_url = $result_link;
   $wm_request->success_url = $success_link;
   $wm_request->fail_url = $fail_link;
   if (get_option('eme_webmoney_demo')) {
      $wm_request->sim_mode = WM_ALL_SUCCESS;
   }
   $wm_request->btn_label = $button_label;
   if (!empty($button_img_url))
      $wm_request->btn_img_url = $button_img_url;

   $form_html = $button_above;
   $form_html .= $wm_request->SetForm(false);
   $form_html .= $button_below;
   return $form_html;
}

function eme_2co_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="2co";
   $eme_2co_business=get_option('eme_2co_business');
   if (!$eme_2co_business) return;

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $success_link = eme_payment_return_url($payment,1);
   $fail_link = eme_payment_return_url($payment,2);
   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   $description=eme_esc_html($description);

   if (get_option('eme_2co_demo')==2)
      $url=CO_SANDBOX_URL;
   else
      $url=CO_LIVE_URL;
   $quantity=1;

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');
   $form_html = $button_above;
   $form_html.="<form action='$url' method='post' name='eme_2co_form' id='eme_2co_form'>";
   $form_html.="<input type='hidden' name='sid' value='$eme_2co_business' />";
   $form_html.="<input type='hidden' name='mode' value='2CO' />";
   $form_html.="<input type='hidden' name='return_url' value='$success_link' />";
   $form_html.="<input type='hidden' name='li_0_type' value='product' />";
   $form_html.="<input type='hidden' name='li_0_product_id' value='$payment_id' />";
   $form_html.="<input type='hidden' name='li_0_name' value='$description' />";
   $form_html.="<input type='hidden' name='li_0_price' value='$price' />";
   $form_html.="<input type='hidden' name='li_0_quantity' value='$quantity' />";
   $form_html.="<input type='hidden' name='currency_code' value='$cur' />";
   if (!empty($button_img_url))
      $form_html.="<input type='image' alt='$button_label' title='$button_label' src='$button_img_url' class='button-primary eme-submit-button' /><br />";
   else
      $form_html.="<input type='submit' value='$button_label' class='button-primary eme-submit-button' /><br />";
   if (get_option('eme_2co_demo')==1) {
      $form_html.="<input type='hidden' name='demo' value='Y' />";
   }
   $form_html.="</form>";
   $form_html.= $button_below;
   return $form_html;
}

function eme_worldpay_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="worldpay";
   $worldpay_instid=get_option('eme_worldpay_instid');
   $worldpay_md5_secret=get_option('eme_worldpay_md5_secret');
   if (!$worldpay_instid) return;

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $success_link = eme_payment_return_url($payment,1);
   $fail_link = eme_payment_return_url($payment,2);
   $notification_link = add_query_arg(array('eme_eventAction'=>'worldpay_notification'),$events_page_link);
   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   if (get_option('eme_worldpay_demo')==1)
      $url=WORLDPAY_SANDBOX_URL;
   else
      $url=WORLDPAY_LIVE_URL;
   $quantity=1;

   $description=eme_esc_html($description);

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');
   $form_html = $button_above;
   $form_html.="<form action='$url' method='post' name='eme_worldpay_form' id='eme_worldpay_form'>";
   $form_html.="<input type='hidden' name='instId' value='$worldpay_instid' />";
   $form_html.="<input type='hidden' name='cartId' value='$payment_id' />";
   $form_html.="<input type='hidden' name='desc' value='$description' />";
   $form_html.="<input type='hidden' name='amount' value='$price' />";
   $form_html.="<input type='hidden' name='currency' value='$cur' />";
   // for worldpay notifications to work: enable dynamic payment response in your worldpay setup, using the param MC_callback
   // also: set the Payment Response password and if wanted, the MD5 secret and field combo
   $form_html.="<input type='hidden' name='MC_callback' value='$notification_link' />";

   if ($worldpay_md5_secret) {
      require_once 'payment_gateways/worldpay/eme-worldpay.php';
      $params_arr=explode(':',get_option('eme_worldpay_md5_parameters'));
      $signature=eme_generate_worldpay_signature($worldpay_md5_secret,$params_arr,$worldpay_instid,$payment_id,$cur,$price);
      $form_html.="<input type='hidden' name='signature' value='$signature' />";
   }

   if (!empty($button_img_url))
      $form_html.="<input type='image' alt='$button_label' title='$button_label' src='$button_img_url' class='button-primary eme-submit-button' /><br />";
   else
      $form_html.="<input type='submit' value='$button_label' class='button-primary eme-submit-button' /><br />";
   if (get_option('eme_worldpay_demo')==1) {
      $form_html.="<input type='hidden' name='testMode' value='100' />";
   }
   $form_html.="</form>";
   $form_html.= $button_below;
   return $form_html;
}

function eme_braintree_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="braintree";
   $eme_braintree_private_key=get_option('eme_braintree_private_key');
   $eme_braintree_public_key=get_option('eme_braintree_public_key');
   $eme_braintree_merchant_id=get_option('eme_braintree_merchant_id');
   $eme_braintree_env=get_option('eme_braintree_env');
   if (!$eme_braintree_public_key) return;

   require_once('payment_gateways/braintree/braintree_php-4.2.0/lib/Braintree.php');
   $braintree_gateway = new Braintree_Gateway(array(
    'environment' => $eme_braintree_env,
    'merchantId' => $eme_braintree_merchant_id,
    'publicKey' => $eme_braintree_public_key,
    'privateKey' => $eme_braintree_private_key
   ));
   $clientToken = $braintree_gateway->clientToken()->generate();

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];

   $quantity=1;

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');
   $form_html = $button_above;
   $form_html.="<form action='' method='post' name='eme_braintree_form' id='eme_braintree_form'>
   <div id='braintree-payment-form-div'></div>
   <input type='hidden' name='payment_id' value='$payment_id' />
   <input type='hidden' name='eme_eventAction' value='braintree_charge' />
   <input type='hidden' name='eme_multibooking' value='$multi_booking' />
   <input type='hidden' name='braintree_nonce' id='braintree_nonce' />
   ";

   if (!empty($button_img_url))
      $form_html.="<input type='image' alt='$button_label' title='$button_label' src='$button_img_url' class='button-primary eme-submit-button' /><br />";
   else
      $form_html.="<input type='button' value='$button_label' id='braintree_submit_button' class='button-primary eme-submit-button' /><br />";
   $form_html.='</form>
<script>
jQuery(document).ready(function ($) {
   $.getScript("https://js.braintreegateway.com/web/dropin/1.10.0/js/dropin.min.js",function () {
	var clientToken = "'.$clientToken.'";
	var form = document.querySelector("#eme_braintree_form");
        braintree.dropin.create({
	  authorization: clientToken,
          container: "#braintree-payment-form-div",
        }, function (createErr, instance) {
          if (createErr) {
            console.log("Create Error", createErr);
            return;
          }
          $("#braintree_submit_button").on("click", function() {
            instance.requestPaymentMethod(function (err, payload) {
              if (err) {
                console.log("Request Payment Method Error", err);
                return;
              }
              // Add the nonce to the form and submit
              $("#braintree_nonce").val(payload.nonce);
              $("#eme_braintree_form").submit();
            });
          });
        });
   });
});
</script>
   ';
   $form_html.= $button_below;
   return $form_html;
}

function eme_stripe_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="stripe";
   $eme_stripe_public_key=get_option('eme_stripe_public_key');
   if (!$eme_stripe_public_key) return;
   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      // stripe doesn't like the single quotes
      $description = preg_replace('/\'/', '', $description);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   // stripe doesn't like the single quotes
   $description = preg_replace('/\'/', '', $description);
   $description=eme_esc_html($description);

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label = htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');

   $form_html = $button_above;
   $form_html .="<form action='' method='post' name='eme_stripe_form' id='eme_stripe_form'>
   <input type='hidden' name='payment_id' value='$payment_id' />
   <input type='hidden' name='eme_eventAction' value='stripe_charge' />
   <input type='hidden' name='description' value='$description' />
   <input type='hidden' name='price' value='$price' />
   <input type='hidden' name='cur' value='$cur' />
   ";
   if (!empty($button_img_url))
         $form_html.="<input type='image' src='$button_img_url' alt='$button_label' title='$button_label' class='button-primary eme-submit-button'/><br />";
   else
         $form_html.="<input type='submit' value='$button_label' class='button-primary eme-submit-button' /><br />";
   $form_html.=wp_nonce_field("$price$cur",'eme_stripe_nonce',false,false);
   $form_html.="</form>";
   $form_html.= $button_below;
   return $form_html;
}

function eme_paymill_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="paymill";
   if (get_option('eme_paymill_demo')==1) {
	   $eme_paymill_public_key=get_option('eme_paymill_public_test_key');
   } else {
	   $eme_paymill_public_key=get_option('eme_paymill_public_key');
   }
   if (!$eme_paymill_public_key) return;
   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      // doesn't like the single quotes
      $description = preg_replace('/\'/', '', $description);
      $member = eme_get_member($payment['member_id']);
      $person = eme_get_person ($member['person_id']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $booking_ids = eme_get_payment_booking_ids($payment['id']);
      $booking=eme_get_booking($booking_ids[0]);
      $person = eme_get_person($booking['person_id']);
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      // doesn't like the single quotes
      $description = preg_replace('/\'/', '', $description);
      $booking_ids = eme_get_payment_booking_ids($payment['id']);
      $booking=eme_get_booking($booking_ids[0]);
      $person = eme_get_person($booking['person_id']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);
   $item_name=esc_attr(get_option('blog_name'));
   if (empty($item_name))
	   $item_name=$description;

   $description=eme_esc_html($description);
   $item_name=eme_esc_html($item_name);

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $quantity=1;

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label = htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');
   if (!empty($button_img_url)) {
	   $data_logo="data-logo='$button_img_url'";
   } else {
	   $data_logo="";
   }

   $locale_code = get_locale();
   $locale_code = preg_replace( "/_/","-", $locale_code );
   // the price for paymill is in cents, but for the button we will convert it back to something sensible
   $button_price = eme_localized_price($price/100,$cur);
   $button_text_inside_form = eme_esc_html(sprintf(__('Pay %s','events-made-easy'),$button_price));
   $person_email = $person['email'];
   $form_html = $button_above;
   $form_html.="<form action='' method='post' name='eme_paymill_form' id='eme_paymill_form'>
    <script
        src='https://button.paymill.com/v1/'
        id='button'
        data-label='$button_label'
        data-title='$item_name'
        data-description='$description'
        data-submit-button='$button_text_inside_form'
        data-amount='$price'
        data-currency='$cur'
        data-public-key='$eme_paymill_public_key'
        data-email='$person_email'
        $data_logo
        >
    </script>
    <script>
    var PAYMILL_PUBLIC_KEY = '$eme_paymill_public_key';
    </script>
    <script src='https://bridge.paymill.com/'></script>
   <input type='hidden' name='payment_id' value='$payment_id' />
   <input type='hidden' name='eme_eventAction' value='paymill_charge' />
   <input type='hidden' name='eme_multibooking' value='$multi_booking' />
   ";
   $form_html.="</form>";
   $form_html.= $button_below;
   return $form_html;
}

function eme_fdgg_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   global $eme_timezone;
   $gateway="fdgg";
   $store_name = get_option('eme_fdgg_store_name');
   $shared_secret = get_option('eme_fdgg_shared_secret');
   if (!$store_name) return;

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $success_link = eme_payment_return_url($payment,1);
   $fail_link = eme_payment_return_url($payment,2);

   // we add the next lines to be conform with the others, but fdgg ignores the description anyway
   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   $description=eme_esc_html($description);

   // the live or sandbox url
   $mode = get_option('eme_fdgg_url');
   if (preg_match('/sandbox|merchanttest/', $mode)) {
           $url = FDGG_SANDBOX_URL;
   } else {
           $url = FDGG_LIVE_URL;
   }

   $quantity=1;
   $datetime=eme_localized_db_date($payment['creation_date'],"Y:m:d-H:i:s");
   $cur_codes=eme_currency_codes();
   $cur_code=$cur_codes[$cur];

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');

   require_once('payment_gateways/fdgg/fdgg-util_sha2.php');
   $form_html = $button_above;
   $form_html.="<form action='$url' method='post' name='eme_fdgg_form' id='eme_fdgg_form'>";
   $form_html.="<input type='hidden' name='timezone' value='$eme_timezone' />";
   $form_html.="<input type='hidden' name='authenticateTransaction' value='false' />";
   $form_html.="<input type='hidden' name='txntype' value='sale' />";
   $form_html.="<input type='hidden' name='mode' value='payonly' />";
   $form_html.="<input type='hidden' name='trxOrigin' value='ECI' />";
   $form_html.="<input type='hidden' name='txndatetime' value='$datetime' />";
   $form_html.="<input type='hidden' name='hash_algorithm' value='SHA512' />";
   $form_html.="<input type='hidden' name='hash' value='".fdgg_createHash($store_name . $datetime . $price . $cur_code . $shared_secret)."' />";
   $form_html.="<input type='hidden' name='storename' value='$store_name' />";
   $form_html.="<input type='hidden' name='chargetotal' value='$price' />";
   $form_html.="<input type='hidden' name='subtotal' value='$price' />";
   $form_html.="<input type='hidden' name='invoicenumber' value='$payment_id' />";
   $form_html.="<input type='hidden' name='oid' value='$payment_id' />";
   $form_html.="<input type='hidden' name='responseSuccessURL' value='$success_link' />";
   $form_html.="<input type='hidden' name='responseFailURL' value='$fail_link' />";
   $form_html.="<input type='hidden' name='eme_eventAction' value='fdgg_notification' />";
   if (!empty($button_img_url))
      $form_html.="<input type='image' src='$button_img_url' alt='$button_label' title='$button_label' class='button-primary eme-submit-button' /><br />";
   else
      $form_html.="<input type='submit' value='$button_label' class='button-primary eme-submit-button' /><br />";
   $form_html.="</form>";
   $form_html.= $button_below;
   return $form_html;
}

function eme_sagepay_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="sagepay";
   $vendor_name = get_option('eme_sagepay_vendor_name');
   if (!$vendor_name) return;

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $success_link = eme_payment_return_url($payment,1);
   $fail_link = eme_payment_return_url($payment,2);
   // sagepay doesn't use a notification url, but sends the status along as part of the return url
   // so we add the notification info to it too, so we can process paid info as usual
   $success_link = add_query_arg(array('eme_eventAction'=>'sagepay_notification'),$success_link);
   $fail_link = add_query_arg(array('eme_eventAction'=>'sagepay_notification'),$fail_link);

   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);


   // the live or sandbox url
   $sagepay_demo = get_option('eme_sagepay_demo');
   if ($sagepay_demo == 1) {
      $sagepay_pwd = get_option('eme_sagepay_test_pwd');
      $url = SAGEPAY_SANDBOX_URL;
   } else {
      $sagepay_pwd = get_option('eme_sagepay_live_pwd');
      $url = SAGEPAY_LIVE_URL;
   }

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');

   $person=array();
   if ($payment['member_id']>0) {
	   $member = eme_get_member($payment['member_id']);
	   if ($member)
              $person = eme_get_person($member['person_id']);
   } else {
      $booking_ids=eme_get_payment_booking_ids($payment_id);
      if ($booking_ids) {
              $booking=eme_get_booking($booking_ids[0]);
              $person = eme_get_person($booking['person_id']);
      }
   }
   if (empty($person))
	   $person=eme_new_person();

   $query = array(
         'VendorTxCode' => $payment_id,
         'Amount' => number_format($price, 2, '.', ''),
         'Currency' => $cur,
         'Description' => $description,
         'SuccessURL' => $success_link,
	 'FailureURL' => $fail_link,
	 'BillingSurname' => $person['lastname'],
	 'BillingFirstnames' => $person['firstname'],
	 'BillingAddress1' => $person['address1'],
	 'BillingCity' => $person['city'],
	 'BillingPostCode' => $person['zip'],
	 'BillingState' => $person['state_code'],
	 'BillingCountry' => $person['country_code'],
	 'DeliverySurname' => $person['lastname'],
	 'DeliveryFirstnames' => $person['firstname'],
	 'DeliveryAddress1' => $person['address1'],
	 'DeliveryCity' => $person['city'],
	 'DeliveryPostCode' => $person['zip'],
	 'DeliveryState' => $person['state_code'],
	 'DeliveryCountry' => $person['country_code']
            );

   require_once 'payment_gateways/sagepay/eme-sagepay-util.php';
   $crypt = SagepayUtil::encryptAes(SagepayUtil::arrayToQueryString($query),$sagepay_pwd);

   $form_html = $button_above;
   $form_html.="<form action='$url' method='post' name='eme_sagepay_form' id='eme_sagepay_form'>";
   $form_html.="<input type='hidden' name='VPSProtocol' value='3.00' />";
   $form_html.="<input type='hidden' name='TxType' value='PAYMENT' />";
   $form_html.="<input type='hidden' name='Vendor' value='$vendor_name' />";
   $form_html.="<input type='hidden' name='Crypt' value='$crypt' />";
   if (!empty($button_img_url))
      $form_html.="<input type='image' src='$button_img_url' alt='$button_label' title='$button_label' class='button-primary eme-submit-button' /><br />";
   else
      $form_html.="<input type='submit' value='$button_label' class='button-primary eme-submit-button' /><br />";
   $form_html.="</form>";
   $form_html.= $button_below;
   return $form_html;
}

function eme_instamojo_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="instamojo";
   $instamojo_key = get_option('eme_instamojo_key');
   $instamojo_auth_token = get_option('eme_instamojo_auth_token');
   if (!$instamojo_key || !$instamojo_auth_token) return;

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $return_link = eme_payment_return_url($payment,'instamojo');
   $notification_link = add_query_arg(array('eme_eventAction'=>'instamojo_notification'),$events_page_link);

   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   $description=eme_esc_html($description);

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');

   $form_html = $button_above;
   $form_html .="<form action='' method='post' name='eme_instamojo_form' id='eme_instamojo_form'>
   <input type='hidden' name='payment_id' value='$payment_id' />
   <input type='hidden' name='eme_eventAction' value='instamojo_charge' />
   <input type='hidden' name='description' value='$description' />
   <input type='hidden' name='price' value='$price' />
   ";
   if (!empty($button_img_url))
         $form_html.="<input type='image' src='$button_img_url' alt='$button_label' title='$button_label' class='button-primary eme-submit-button'/><br />";
   else
         $form_html.="<input type='submit' value='$button_label' class='button-primary eme-submit-button' /><br />";
   $form_html.=wp_nonce_field($price,'eme_instamojo_nonce',false,false);
   $form_html.= $button_below;
   $form_html.="</form>";
   return $form_html;
}

function eme_mollie_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway="mollie";
   $mollie_api_key = get_option('eme_mollie_api_key');
   if (!$mollie_api_key) return;

   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      // mollie doesn't like the single quotes
      $description = preg_replace('/\'/', '', $description);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      // mollie doesn't like the single quotes
      $description = preg_replace('/\'/', '', $description);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   $description=eme_esc_html($description);

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label = htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');

   $form_html = $button_above;
   $form_html .="<form action='' method='post' name='eme_mollie_form' id='eme_mollie_form'>
   <input type='hidden' name='payment_id' value='$payment_id' />
   <input type='hidden' name='eme_eventAction' value='mollie_charge' />
   <input type='hidden' name='description' value='$description' />
   <input type='hidden' name='price' value='$price' />
   <input type='hidden' name='cur' value='$cur' />
   ";
   if (!empty($button_img_url))
         $form_html.="<input type='image' src='$button_img_url' alt='$button_label' title='$button_label' class='button-primary eme-submit-button'/><br />";
   else
         $form_html.="<input type='submit' value='$button_label' class='button-primary eme-submit-button' /><br />";
   $form_html.=wp_nonce_field("$price$cur",'eme_mollie_nonce',false,false);
   $form_html.="</form>";
   $form_html.= $button_below;
   return $form_html;
}

function eme_paypal_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway='paypal';
   $eme_paypal_clientid = get_option('eme_paypal_clientid');
   $eme_paypal_secret = get_option('eme_paypal_secret');
   if (!$eme_paypal_clientid || !$eme_paypal_secret) return;

   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $success_link = eme_payment_return_url($payment,'paypal');
   $fail_link = eme_payment_return_url($payment,2);
   $notification_link = add_query_arg(array('eme_eventAction'=>'paypal_notification'),$events_page_link);

   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   $description=eme_esc_html($description);

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway); 
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');
   $form_html= $button_above;
   $form_html.="<form action='' method='post' name='eme_paypal_form' id='eme_paypal_form'>
   <input type='hidden' name='payment_id' value='$payment_id' />
   <input type='hidden' name='eme_eventAction' value='paypal_charge' />
   <input type='hidden' name='description' value='$description' />
   <input type='hidden' name='price' value='$price' />
   <input type='hidden' name='cur' value='$cur' />
   ";
   if (!empty($button_img_url))
         $form_html.="<input type='image' src='$button_img_url' alt='$button_label' title='$button_label' class='button-primary eme-submit-button' /><br />";
   else
         $form_html.="<input type='submit' value='$button_label' class='button-primary eme-submit-button' /><br />";
   $form_html.=wp_nonce_field("$price$cur",'eme_paypal_nonce',false,false);
   $form_html.="</form>";
   $form_html.= $button_below;
   return $form_html;
}

function eme_legacypaypal_form($line,$payment,$baseprice,$cur,$lang,$multi_booking=0) {
   $gateway='legacypaypal';
   $eme_paypal_business = get_option('eme_legacypaypal_business');
   if (!$eme_paypal_business) return;

   $quantity=1;
   $price=eme_payment_gateway_total($baseprice,$cur,$gateway);
   $events_page_link = eme_get_events_page();
   $payment_id=$payment['id'];
   $success_link = eme_payment_return_url($payment,1);
   $fail_link = eme_payment_return_url($payment,2);
   $notification_link = add_query_arg(array('eme_eventAction'=>'legacypaypal_notification'),$events_page_link);

   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$line['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$line['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   $description=eme_esc_html($description);

   $charge = eme_payment_gateway_extra_charge($baseprice,$gateway);
   $button_above = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_above'),$charge,$cur,$lang);
   $button_label = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_label'),$charge,$cur,$lang);
   if (empty($button_label)) $button_label=$gateway;
   $button_label=htmlentities($button_label);
   $button_below = eme_replace_payment_gateway_placeholders(get_option('eme_'.$gateway.'_button_below'),$charge,$cur,$lang);
   $button_img_url = get_option('eme_'.$gateway.'_button_img_url');

   require_once "payment_gateways/paypal_legacy/Paypal.php";
   $p = new Paypal;

   // the paypal or paypal sandbox url
   $mode = get_option('eme_legacypaypal_url');
   if (preg_match('/sandbox/', $mode)) {
           $p->paypal_url = PAYPAL_SANDBOX_URL;
   } else {
           $p->paypal_url = PAYPAL_LIVE_URL;
   }

   // the timeout in seconds before the button form is submitted to paypal
   // this needs the included addevent javascript function
   // 0 = no delay
   // false = disable auto submission
   $p->timeout = false;

   // the button label
   // false to disable button (if you want to rely only on the javascript auto-submission) not recommended
   $p->button = $button_label;
   if (!empty($button_img_url))
      $p->button_img_url = $button_img_url;

   // no encryption, use the new paypal for that
   $p->encrypt = false;

   // the actual button parameters
   // https://www.paypal.com/IntegrationCenter/ic_std-variable-reference.html
   $p->add_field('charset','utf-8');
   $p->add_field('business', $eme_paypal_business);
   $p->add_field('return', $success_link);
   $p->add_field('cancel_return', $fail_link);
   $p->add_field('notify_url', $notification_link);
   $p->add_field('item_name', $description);
   $p->add_field('item_number', $payment_id);
   $p->add_field('custom', $payment_id);
   $p->add_field('currency_code',$cur);
   $p->add_field('amount', $price);
   $p->add_field('quantity', $quantity);
   $p->add_field('no_shipping', 1);
   if (get_option('eme_legacypaypal_no_tax')) {
      $p->add_field('tax', 0);
   }

   $form_html = $button_above;
   $form_html .= $p->get_button();
   $form_html .= $button_below;
   return $form_html;
}

function eme_complete_instamojo_transaction($payment,$payment_requestid) {
   $instamojo_key = get_option('eme_instamojo_key');
   $instamojo_auth_token = get_option('eme_instamojo_auth_token');
   require_once 'payment_gateways/instamojo/autoload.php';
   $mode = get_option('eme_instamojo_env');
   if (preg_match('/sandbox/', $mode)) {
           $api = new Instamojo\Instamojo($instamojo_key,$instamojo_auth_token,'https://test.instamojo.com/api/1.1/');
   } else {
           $api = new Instamojo\Instamojo($instamojo_key,$instamojo_auth_token);
   }

   $instamojo_requestid=eme_sanitize_request($_GET['payment_request_id']);
   $instamojo_paymentid=eme_sanitize_request($_GET['payment_id']);
   // small sanity check
   if ($instamojo_requestid != $payment_requestid)
	   return;
   // now check the state
   try {
	   $response = $api->paymentRequestPaymentStatus($instamojo_requestid, $instamojo_paymentid);
	   if ($response['payment']['status'] == 'Credit') {
		   eme_update_payment_paid($payment['id'],1,"instamojo",$instamojo_requestid);
	   }
   }
   catch (Exception $e) {
	   return;
   }
}

function eme_complete_stripe_transaction($payment,$stripe_sessionid) {
   $eme_stripe_private_key=get_option('eme_stripe_private_key');

   require_once('payment_gateways/stripe/stripe-php-6.41.0/init.php');
   \Stripe\Stripe::setApiKey("$eme_stripe_private_key");
   $stripe_session = \Stripe\Checkout\Session::retrieve($stripe_sessionid);
   // we don't trust the payment given, we use the one in the session from stripe
   $payment_id=$stripe_session->client_reference_id;
   $stripe_pi_id=$stripe_session->payment_intent;
   if ($payment_id==$payment['id']) {
	   $stripe_pi = \Stripe\PaymentIntent::retrieve($stripe_pi_id);
           if ($stripe_pi->status=='succeeded')
		   eme_update_payment_paid($payment_id,1,"stripe",$stripe_sessionid);
   }
}

function eme_complete_paypal_transaction($payment,$paypal_orderid) {
   require_once "payment_gateways/paypal/autoload.php";
   // the paypal or paypal sandbox url
   $mode = get_option('eme_paypal_url');
   if (preg_match('/sandbox/', $mode)) {
           require_once "payment_gateways/paypal/client_sandbox.php";
           $client = PayPalClient::client();
   } else {
           require_once "payment_gateways/paypal/client_prod.php";
           $client = PayPalClient::client();
   }

   $request = new \PayPalCheckoutSdk\Orders\OrdersCaptureRequest($paypal_orderid);
   $request->prefer('return=representation');
   try {
	   // Call API with your client and get a response for your call
	   $response = $client->execute($request);
	   if ($response->result->status=="COMPLETED") {
		   eme_update_payment_paid($payment['id'],1,"paypal",$paypal_orderid);
		   return 1;
	   } else {
		   return 0;
	   }
   }catch (HttpException $ex) {
	   return 0;
   }
}


function eme_instamojo_notification() {
   global $wpdb;

   $instamojo_key = get_option('eme_instamojo_key');
   $instamojo_auth_token = get_option('eme_instamojo_auth_token');
   $instamojo_salt = get_option('eme_instamojo_salt');
   require_once 'payment_gateways/instamojo/autoload.php';
   $mode = get_option('eme_instamojo_env');
   if (preg_match('/sandbox/', $mode)) {
           $api = new Instamojo\Instamojo($instamojo_key,$instamojo_auth_token,'https://test.instamojo.com/api/1.1/');
   } else {
           $api = new Instamojo\Instamojo($instamojo_key,$instamojo_auth_token);
   }


   $instamojo_requestid=eme_sanitize_request($_POST['payment_request_id']);
   $instamojo_paymentid=eme_sanitize_request($_POST['payment_id']);

   $table_name = $wpdb->prefix.PAYMENTS_TBNAME;
   $sql = $wpdb->prepare("SELECT id FROM $table_name WHERE pg_pids=%s",$instamojo_requestid);
   $payment_id = $wpdb->get_col($sql);
   $payment = eme_get_payment($payment_id);

   // code from https://docs.instamojo.com/docs/payments-api
   $data = $_POST;
   $mac_provided = $data['mac'];  // Get the MAC from the POST data
   unset($data['mac']);  // Remove the MAC key from the data.
   $ver = explode('.', phpversion());
   ksort($data, SORT_STRING | SORT_FLAG_CASE);
   $mac_calculated = hash_hmac("sha1", implode("|", $data), $instamojo_salt);
   if ($mac_provided == $mac_calculated && $payment['pg_pids']==$instamojo_requestid) {
	   if ($data['status'] == "Credit") {
		   eme_update_payment_paid($payment_id,1,"instamojo",$instamojo_requestid);
	   }
   }
}

function eme_legacypaypal_notification() {
   require_once 'payment_gateways/paypal_legacy/IPN.php';
   $ipn = new IPN;

   // the paypal url, or the sandbox url, or the ipn test url
   //$ipn->paypal_url = 'https://www.paypal.com/cgi-bin/webscr';
   $mode = get_option('eme_legacypaypal_url');
   if (preg_match('/sandbox/', $mode)) {
	   $ipn->paypal_url = PAYPAL_IPN_SANDBOX_URL;
   } else {
	   $ipn->paypal_url = PAYPAL_IPN_LIVE_URL;
   }

   // your paypal email (the one that receives the payments)
   $ipn->paypal_email = get_option('eme_legacypaypal_business');

   // log to file options
   $ipn->log_to_file = false;			// write logs to file
   $ipn->log_filename = '/path/to/ipn.log';  	// the log filename (should NOT be web accessible and should be writable)

   // log to e-mail options
   $ipn->log_to_email = false;		// send logs by e-mail
   $ipn->log_email = '';		// where you want to receive the logs
   $ipn->log_subject = 'IPN Log: ';	// prefix for the e-mail subject

   // array of currencies accepted or false to disable
   //$ipn->currencies = array('USD','EUR');
   $ipn->currencies = false;

   // date format on log headers (default: dd/mm/YYYY HH:mm:ss)
   // see http://php.net/date
   $ipn->date_format = 'd/m/Y H:i:s';

   // Prefix for file and mail logs
   $ipn->pretty_ipn = "IPN Values received:\n\n";

   // configuration ended, do the actual check

   if($ipn->ipn_is_valid()) {
      /*
         A valid ipn was received and passed preliminary validations
         You can now do any custom validations you wish to ensure the payment was correct
         You can access the IPN data with $ipn->ipn['value']
         The complete() method below logs the valid IPN to the places you choose
       */
      $payment_id=intval($ipn->ipn['custom']);
      eme_update_payment_paid($payment_id,1,"paypal",$ipn->txn_id);
      $ipn->complete();
   }
}

function eme_2co_notification() {
   $business=get_option('eme_2co_business');
   $secret=get_option('eme_2co_secret');

   if ($_POST['message_type'] == 'ORDER_CREATED'
       || $_POST['message_type'] == 'INVOICE_STATUS_CHANGED') {
      $insMessage = array();
      foreach ($_POST as $k => $v) {
         $insMessage[$k] = $v;
      }
 
      $hashSid = $insMessage['vendor_id'];
      if ($hashSid != $business) {
         die ('Not the 2Checkout Account number it should be ...');
      }
      $hashOrder = $insMessage['sale_id'];
      $hashInvoice = $insMessage['invoice_id'];
      $StringToHash = strtoupper(md5($hashOrder . $hashSid . $hashInvoice . $secret));
 
      if ($StringToHash != $insMessage['md5_hash']) {
	 die(__('Hash Incorrect','events-made-easy'));
      }

      if ($insMessage['invoice_status'] == 'approved' || $insMessage['invoice_status'] == 'deposited') {
         $payment_id=intval($insMessage['item_id_1']);
         eme_update_payment_paid($payment_id,1,"2co",$hashInvoice);
      }
   }
}

function eme_webmoney_notification() {
   $webmoney_purse = get_option('eme_webmoney_purse');
   $webmoney_secret = get_option('eme_webmoney_secret');

   require_once('payment_gateways/webmoney/webmoney.inc.php');
   $wm_notif = new WM_Notification(); 
   if ($wm_notif->GetForm() != WM_RES_NOPARAM) {
      $amount=$wm_notif->payment_amount;
      if ($webmoney_purse != $wm_notif->payee_purse) {
         die ('Not the webmoney purse it should be ...');
      }
      #if ($price != $amount) {
      #   die ('Not the webmoney amount I expected ...');
      #}
      $payment_id=intval($wm_notif->payment_no);
      if ($wm_notif->CheckMD5($webmoney_purse, $amount, $payment_id, $webmoney_secret) == WM_RES_OK) {
         eme_update_payment_paid($payment_id,1,"webmoney",$wm_notif->sys_invs_no);
      }
   }
}

function eme_fdgg_notification() {
   $store_name = get_option('eme_fdgg_store_name');
   $shared_secret = get_option('eme_fdgg_shared_secret');
   require_once('payment_gateways/fdgg/fdgg-util_sha2.php');

   $payment_id      = intval($_POST['invoicenumber']);
   $charge_total    = $_POST['charge_total'];
   $approval_code   = $_POST['approval_code'];
   $response_hash   = $_POST['response_hash'];
   $response_status = $_POST['status'];

   // First Data only allows USD
   $payment=eme_get_payment($payment_id);
   if ($payment['member_id']>0) {
      $member = eme_get_member($payment['member_id']);
      if ($member) {
	      $person = eme_get_person($member['person_id']);
	      $membership=eme_get_membership($member['membership_id']);
	      $cur = $membership['properties']['currency'];
      } else {
	      _e('Incorrect payment id.','events-made-easy');return;
      }
   } else {
      $booking_ids=eme_get_payment_booking_ids($payment_id);
      if ($booking_ids) {
	      $booking=eme_get_booking($booking_ids[0]);
	      $event=eme_get_event($booking['event_id']);
	      $person = eme_get_person($booking['person_id']);
	      $cur=$event['currency'];
	      $multi_booking = isset($_POST['eme_multibooking']) ? intval($_POST['eme_multibooking']) : 0;
      } else {
	      _e('Incorrect payment id.','events-made-easy');return;
      }
   }
   $datetime=eme_localized_db_date($payment['creation_date'],"Y:m:d-H:i:s");
   $cur_codes=eme_currency_codes();
   $cur_code=$cur_codes[$cur];
   $calc_hash=fdgg_createHash($shared_secret.$approval_code.$charge_total.$cur_code.$datetime.$store_name);

   if ($response_hash != $calc_hash) {
      die(__('Hash Incorrect','events-made-easy'));
   }

   // TODO: do some extra checks, like the price paid and such
   #$price=eme_get_total_booking_price($booking);

   if (strtolower($response_status) == 'approved') {
      eme_update_payment_paid($payment_id,1,"fdgg",$payment_id);
   }
}

function eme_stripe_notification() {
   $eme_stripe_private_key=get_option('eme_stripe_private_key');
   $webhook_secret=get_option('eme_stripe_webhook_secret');

   require_once('payment_gateways/stripe/stripe-php-6.41.0/init.php');
   \Stripe\Stripe::setApiKey("$eme_stripe_private_key");

   $payload = @file_get_contents('php://input');
   $sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
   $event = null;

   // verify the signature
   try {
	   $event = \Stripe\Webhook::constructEvent(
		   $payload, $sig_header, $webhook_secret
	   );
   } catch(\UnexpectedValueException $e) {
	   // Invalid payload
	   http_response_code(400);
	   return;
   } catch(\Stripe\Error\SignatureVerification $e) {
	   // Invalid signature
	   http_response_code(400);
	   return;
   }

   // Handle the checkout.session.completed event
   if ($event->type == 'checkout.session.completed') {
	   $stripe_session = $event->data->object;
	   $payment_id=$stripe_session->client_reference_id;
	   eme_update_payment_paid($payment_id,1,"stripe",$stripe_sessionid);
   }
   http_response_code(200);
}

function eme_stripe_webhook() {
   $eme_stripe_private_key=get_option('eme_stripe_private_key');
   if (empty($eme_stripe_private_key)) {
	   return;
   }
   // do nothing if the events page is on localhost
   $events_page_link = eme_get_events_page();
   $notification_link = add_query_arg(array('eme_eventAction'=>'stripe_notification'),$events_page_link);
   if (strstr($events_page_link,'localhost')) {
	   update_option('eme_stripe_webhook_error',__('since this is a site running on localhost, no webhook will be created','events-made-easy'));
	   return;
   }

   require_once('payment_gateways/stripe/stripe-php-6.41.0/init.php');
   \Stripe\Stripe::setApiKey("$eme_stripe_private_key");

   // first delete all webhooks
   $webhooks = null;
   try {
	   $webhooks = \Stripe\WebhookEndpoint::all(["limit" => 100]);
	   if (!empty($webhooks)) {
		   foreach ($webhooks->data as $webhook) {
			   $endpoint_id=$webhook->id;
			   $endpoint = \Stripe\WebhookEndpoint::retrieve("$endpoint_id");
			   $endpoint->delete();
		   }
	   }
   } catch(\Stripe\Error\InvalidRequest $e) {
           $body = $e->getJsonBody();
           $err  = $body['error'];
           update_option('eme_stripe_webhook_error',$err['message']);
           return;
   }

   update_option('eme_stripe_webhook_secret','');
   try {
	   $endpoint = \Stripe\WebhookEndpoint::create([
		   "url" => $notification_link,
		   "enabled_events" => ["checkout.session.completed"]
	   ]);
   } catch(\Stripe\Error\InvalidRequest $e) {
	   $body = $e->getJsonBody();
	   $err  = $body['error'];
	   update_option('eme_stripe_webhook_error',$err['message']);
	   return;
   }
   update_option('eme_stripe_webhook_secret',$endpoint->secret);
   update_option('eme_stripe_webhook_error','');
}

function eme_paypal_charge() {
   $events_page_link = eme_get_events_page();
   $payment_id=intval($_POST['payment_id']);
   $price=eme_sanitize_request($_POST['price']);
   $cur=eme_sanitize_request($_POST['cur']);
   $description=eme_sanitize_request($_POST['description']);
   $payment=eme_get_payment($payment_id);
   $success_link = eme_payment_return_url($payment,"paypal");
   $fail_link = eme_payment_return_url($payment,2);
   $notification_link = add_query_arg(array('eme_eventAction'=>'paypal_notification'),$events_page_link);

   // no cheating
   if (!wp_verify_nonce($_POST['eme_paypal_nonce'],"$price$cur")) {
	   header("Location: $fail_link");exit;
   }

   require_once "payment_gateways/paypal/autoload.php";
   // the paypal or paypal sandbox url
   $mode = get_option('eme_paypal_url');
   if (preg_match('/sandbox/', $mode)) {
           require_once "payment_gateways/paypal/client_sandbox.php";
           $client = PayPalClient::client();
   } else {
           require_once "payment_gateways/paypal/client_prod.php";
           $client = PayPalClient::client();
   }

   // although mentioning items is not obligated, you need it or on the paypal window the amount and description won't show
   $request = new \PayPalCheckoutSdk\Orders\OrdersCreateRequest();
   $request->prefer('return=representation');
   $request->body = [
           "intent" => "CAPTURE",
           "purchase_units" => [[
                   "reference_id" => $payment_id,
		   "description" => "$description",
                   "amount" => [
                           "value" => "$price",
			   "currency_code" => "$cur",
			   "breakdown" => [
				   'item_total' => [
					   "value" => "$price",
					   "currency_code" => "$cur"
				   ]
			   ]
		   ],
		   "items" => [[
                           "name" => "$description",
                           "description" => "$description",
			   "unit_amount" => [
				   "value" => "$price",
				   "currency_code" => "$cur"
			   ],
			   "quantity" => "1"
		   ]]
           ]],
           "application_context" => [
                   "cancel_url" => $fail_link,
                   "return_url" => $success_link
           ]
   ];

   $url = "";
   try {
           // Call API with your client and get a response for your call
           $response = $client->execute($request);
           // If call returns body in response, you can get the deserialized version from the result attribute of the response
	   foreach ($response->result->links as $link) {
		   if ($link->rel=='approve')
			   $url=$link->href;
	   }
   }catch (HttpException $ex) {
           return "Paypal API call failed: ".$ex->statusCode.': ' . htmlspecialchars($ex->getMessage());
   }
   if (!empty($url)) {
	   // we'll store the paypal payment id already, so when people arrive to the redirecturl before the webhook fired, we can check for it
	   eme_update_payment_pg_pids($payment_id,$response->result->id);
	   header("Location: $url");exit;
   }
}

function eme_paymill_charge() {
   $gateway="paymill";
   if (get_option('eme_paymill_demo')==1) {
	   $eme_paymill_private_key=get_option('eme_paymill_private_test_key');
   } else {
	   $eme_paymill_private_key=get_option('eme_paymill_private_key');
   }
   $payment_id = intval($_POST['payment_id']);
   $payment = eme_get_payment($payment_id);
   $member_payment=0;
   $multi_booking = 0;
   if ($payment['member_id']>0) {
      $member = eme_get_member($payment['member_id']);
      if ($member) {
	      $person = eme_get_person($member['person_id']);
	      $membership=eme_get_membership($member['membership_id']);
	      $cur = $membership['properties']['currency'];
      } else {
	      _e('Incorrect payment id.','events-made-easy');return;
      }
   } else {
      $booking_ids=eme_get_payment_booking_ids($payment_id);
      if ($booking_ids) {
	      $booking=eme_get_booking($booking_ids[0]);
	      $event=eme_get_event($booking['event_id']);
	      $person = eme_get_person($booking['person_id']);
	      $cur=$event['currency'];
	      $multi_booking = isset($_POST['eme_multibooking']) ? intval($_POST['eme_multibooking']) : 0;
      } else {
	      _e('Incorrect payment id.','events-made-easy');return;
      }
   }

   $events_page_link = eme_get_events_page();
   $success_link = eme_payment_return_url($payment,1);
   $fail_link = eme_payment_return_url($payment,2);
   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$membership['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = __("Multiple booking request",'events-made-easy');
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = sprintf(__("Booking for '%s'",'events-made-easy'),$event['event_name']);
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   if ($payment['member_id']>0) {
	   $price = eme_get_member_payment_price($payment_id);
   } else {
	   $price = eme_get_payment_price($payment_id);
   }
   $price=eme_payment_gateway_total($price,$cur,'paymill');
   require_once('payment_gateways/paymill/autoload.php');
   try {
	   if (!isset($_POST['paymillToken']))
		   throw new Exception("The Paymill Token was not generated correctly");
	   // price needs to be in cents, but eme_payment_gateway_total handles that
	   $request = new Paymill\Request($eme_paymill_private_key);
	   $transaction = new Paymill\Models\Request\Transaction();
	   $transaction->setAmount($price) // e.g. "4200" for 42.00 EUR
                ->setCurrency($cur)
                ->setToken($_POST['paymillToken'])
                ->setDescription($description);
	   $response = $request->create($transaction);
	   if ($response->getStatus() == 'closed') {
		   eme_update_payment_paid($payment_id,1,"paymill",$response->getShortId());
		   header("Location: $success_link");exit;
	   } else {
		   header("Location: $fail_link");exit;
	   }
   } catch(\Paymill\Services\PaymillException $e){
	   // The card has been declined
	   $error = $e->getErrorMessage();
	   die($error);
   }
}

function eme_stripe_charge() {
   $events_page_link = eme_get_events_page();
   $payment_id=intval($_POST['payment_id']);
   $price=eme_sanitize_request($_POST['price']);
   $cur=eme_sanitize_request($_POST['cur']);
   $description=eme_sanitize_request($_POST['description']);
   $payment=eme_get_payment($payment_id);
   $success_link = eme_payment_return_url($payment,"stripe");
   $fail_link = eme_payment_return_url($payment,2);

   // no cheating
   if (!wp_verify_nonce($_POST['eme_stripe_nonce'],"$price$cur")) {
	   header("Location: $fail_link");exit;
   }

   $eme_stripe_private_key=get_option('eme_stripe_private_key');
   $eme_stripe_public_key=get_option('eme_stripe_public_key');
   $item_name=esc_attr(get_option('blog_name'));
   if (empty($item_name))
	   $item_name=$description;

   require_once('payment_gateways/stripe/stripe-php-6.41.0/init.php');
   \Stripe\Stripe::setApiKey("$eme_stripe_private_key");
   \Stripe\Stripe::setAppInfo(
	"WordPress Events Made Easy Stripe plugin",
	"2.1.18",
	"https://www.e-dynamics.be"
   );
   $stripe_session = \Stripe\Checkout\Session::create([
	   'payment_method_types' => ['card'],
	   'line_items' => [[
		   'name' => $item_name,
		   'description' => $description,
		   'amount' => $price,
		   'currency' => strtolower($cur),
		   'quantity' => 1,
	   ]],
	   'client_reference_id' => $payment_id,
	   'success_url' => $success_link,
	   'cancel_url' => $fail_link
   ]);

   $stripe_session_id=$stripe_session->id;
   eme_update_payment_pg_pids($payment_id,$stripe_session_id);
   print "<html><body>
<script src='https://js.stripe.com/v3/'></script>
<script>
      var stripe = Stripe('$eme_stripe_public_key');
      stripe.redirectToCheckout({
         sessionId: '$stripe_session_id'
      });
</script>
</body></html>
   ";
}

function eme_braintree_charge() {
   $gateway="braintree";
   $eme_braintree_private_key=get_option('eme_braintree_private_key');
   $eme_braintree_public_key=get_option('eme_braintree_public_key');
   $eme_braintree_merchant_id=get_option('eme_braintree_merchant_id');
   $eme_braintree_env=get_option('eme_braintree_env');
   $payment_id = intval($_POST['payment_id']);
   $payment = eme_get_payment($payment_id);
   $multi_booking = 0;
   if ($payment['member_id']>0) {
      $member = eme_get_member($payment['member_id']);
      if ($member) {
	      $person = eme_get_person($member['person_id']);
	      $membership=eme_get_membership($member['membership_id']);
	      $cur = $membership['properties']['currency'];
      } else {
	      _e('Incorrect payment id.','events-made-easy');return;
      }
   } else {
      $booking_ids=eme_get_payment_booking_ids($payment_id);
      if ($booking_ids) {
	      $booking=eme_get_booking($booking_ids[0]);
	      $event=eme_get_event($booking['event_id']);
	      $person = eme_get_person($booking['person_id']);
	      $cur=$event['currency'];
	      $multi_booking = isset($_POST['eme_multibooking']) ? intval($_POST['eme_multibooking']) : 0;
      } else {
	      _e('Incorrect payment id.','events-made-easy');return;
      }
   }

   $success_link = eme_payment_return_url($payment,1);
   $fail_link = eme_payment_return_url($payment,2);

   // we add the next lines to be conform with the others, but braintree ignores the description anyway
   if ($payment['member_id']>0) {
      $description = sprintf(__("Member signup for '%s'",'events-made-easy'),$membership['name']);
      $filtername='eme_member_paymentform_description_filter';
   } elseif ($multi_booking) {
      $description = esc_attr(__("Multiple booking request",'events-made-easy'));
      $filtername='eme_rsvp_paymentform_description_filter';
   } else {
      $description = esc_attr(sprintf(__("Booking for '%s'",'events-made-easy'),$event['event_name']));
      $filtername='eme_rsvp_paymentform_description_filter';
   }
   if (has_filter($filtername))
      $description=apply_filters($filtername,$description,$payment,$gateway);

   if ($payment['member_id']>0) {
	   $price = eme_get_member_payment_price($payment_id);
   } else {
	   $price = eme_get_payment_price($payment_id);
   }
   $price=eme_payment_gateway_total($price,$cur,'braintree');
   require_once('payment_gateways/braintree/braintree_php-4.2.0/lib/Braintree.php');
   if (!isset($_POST['braintree_nonce']))
	   die("The nonce was not generated correctly");
   $braintree_gateway = new Braintree_Gateway(array(
    'environment' => $eme_braintree_env,
    'merchantId' => $eme_braintree_merchant_id,
    'publicKey' => $eme_braintree_public_key,
    'privateKey' => $eme_braintree_private_key
   ));
   $result = $braintree_gateway->transaction()->sale(array(
			   "amount" => $price,
			   "paymentMethodNonce" => $_POST['braintree_nonce'],
			   "orderId" => $payment_id
			   ));
   if ($result->success) {
	   $transaction = $result->transaction;
	   eme_update_payment_paid($payment_id,1,"braintree",$transaction->id);
      header("Location: $success_link");exit;
   } else {
      header("Location: $fail_link");exit;
	   die(__('Your payment failed.','events-made-easy'));
   }
}

function eme_instamojo_charge() {
   $gateway="instamojo";
   $instamojo_key = get_option('eme_instamojo_key');
   $instamojo_auth_token = get_option('eme_instamojo_auth_token');
   if (!$instamojo_key || !$instamojo_auth_token) return;

   $events_page_link = eme_get_events_page();
   $payment_id=intval($_POST['payment_id']);
   $price=eme_sanitize_request($_POST['price']);
   $description=eme_sanitize_request($_POST['description']);
   $payment=eme_get_payment($payment_id);

   $return_link = eme_payment_return_url($payment,'instamojo');
   $fail_link = eme_payment_return_url($payment,2);
   $notification_link = add_query_arg(array('eme_eventAction'=>'instamojo_notification'),$events_page_link);

   // no cheating
   if (!wp_verify_nonce($_POST['eme_instamojo_nonce'],$price)) {
	   header("Location: $fail_link");exit;
   }

   require_once 'payment_gateways/instamojo/autoload.php';
   $mode = get_option('eme_instamojo_env');
   if (preg_match('/sandbox/', $mode)) {
	   $api = new Instamojo\Instamojo($instamojo_key,$instamojo_auth_token,'https://test.instamojo.com/api/1.1/');
   } else {
	   $api = new Instamojo\Instamojo($instamojo_key,$instamojo_auth_token);
   }
   try {
	   $instamojo_payment = $api->paymentRequestCreate(array(
		   "purpose" => $description,
		   "amount" => "$price",
		   "redirect_url" => $return_link,
		   "webhook" => $notification_link,

	   ));
	   $url=$instamojo_payment['longurl'];
   }
   catch (Exception $e) {
	   $url = "";
	   print "Instamojo API call failed: " . htmlspecialchars($e->getMessage());
   }

   if (!empty($url)) {
      // we'll store the mollie payment id already, so when people arrive to the redirecturl before the webhook fired, we can check for it
      eme_update_payment_pg_pids($payment['id'],$instamojo_payment['id']);
      header("Location: $url");exit;
   }
}

function eme_mollie_charge() {
   $events_page_link = eme_get_events_page();
   $payment_id=intval($_POST['payment_id']);
   $price=eme_sanitize_request($_POST['price']);
   $cur=eme_sanitize_request($_POST['cur']);
   $description=eme_sanitize_request($_POST['description']);
   $payment=eme_get_payment($payment_id);

   $mollie_api_key = get_option('eme_mollie_api_key');
   $return_link = eme_payment_return_url($payment,'mollie');
   $fail_link = eme_payment_return_url($payment,'mollie');
   $notification_link = add_query_arg(array('eme_eventAction'=>'mollie_notification'),$events_page_link);

   // no cheating
   if (!wp_verify_nonce($_POST['eme_mollie_nonce'],"$price$cur")) {
	   header("Location: $fail_link");exit;
   }

   // Avoid loading the Mollie API if it is already loaded by another plugin
   if (!class_exists('Mollie\Api\MollieApiClient')) {
	   require_once 'payment_gateways/Mollie/2.10.0/vendor/autoload.php';
   }
   $mollie = new \Mollie\Api\MollieApiClient();
   $mollie->setApiKey($mollie_api_key);

   // Mollie needs the price in EUR and 2 decimals
   try {
      $mollie_payment = $mollie->payments->create(
            array(
               'amount'      => array("currency"=>$cur, "value"=>sprintf("%01.2f",$price)),
               'description' => $description,
               'redirectUrl' => $return_link,
               'webhookUrl'  => $notification_link,
               'metadata'    => array(
                  'payment_id' => $payment_id
                  )
               )
            );
      $url = $mollie_payment->getCheckoutUrl();
   }
   catch (\Mollie\Api\Exceptions\ApiException $e) {
      $url="";
      print "Mollie API call failed: " . htmlspecialchars($e->getMessage()) . " on field " . htmlspecialchars($e->getField());
   }

   if (!empty($url)) {
      // we'll store the mollie payment id already, so when people arrive to the redirecturl before the webhook fired, we can check for it
      eme_update_payment_pg_pids($payment['id'],$mollie_payment->id);
      header("Location: $url");exit;
   }
}

function eme_mollie_notification($mollie_payment_id=0) {
   $api_key = get_option('eme_mollie_api_key');
   // Avoid loading the Mollie API if it is already loaded by another plugin
   if (!class_exists('Mollie\Api\MollieApiClient')) {
	   require_once 'payment_gateways/Mollie/2.10.0/vendor/autoload.php';
   }

   $mollie = new \Mollie\Api\MollieApiClient();
   $mollie->setApiKey($api_key);
   if (!$mollie_payment_id)
	   $mollie_payment_id = eme_sanitize_request($_POST["id"]);
   $mollie_payment = $mollie->payments->get($mollie_payment_id);
   $payment_id = $mollie_payment->metadata->payment_id;
   $payment = eme_get_payment($payment_id);
   if ($payment['pg_pids'] != $mollie_payment_id) {
	   return;
   }
   if ($mollie_payment->isPaid()) {
      eme_update_payment_paid($payment_id,1,"mollie",$mollie_payment->id);
   }
}

function eme_worldpay_notification() {
   // for worldpay notifications to work: enable dynamic payment response in your worldpay setup, using the param MC_callback
   $worldpay_demo = get_option('eme_worldpay_demo');
   if ($worldpay_demo == 1) {
      $worldpay_pwd = get_option('eme_worldpay_test_pwd');
   } else {
      $worldpay_pwd = get_option('eme_worldpay_live_pwd');
   }

   $post_pwd=$_POST["callbackPW"];
   $trans_status=$_POST["transStatus"];
   $trans_id=$_POST["transId"];
   $test_mode=isset($_POST ['testMode']) ? $_POST ['testMode'] : 0;
   $post_instid=$_POST["instId"];
   $worldpay_instid=get_option('eme_worldpay_instid');
   $payment_id=intval($_POST["cartId"]);
   if ($post_pwd==$worldpay_pwd && $trans_status=='Y' && $test_mode==0 && $post_instid==$worldpay_instid) {
      eme_update_payment_paid($payment_id,1,"worldpay",$trans_id);
   }
}

function eme_sagepay_notification() {
   $sagepay_demo = get_option('eme_sagepay_demo');
   if ($sagepay_demo == 1) {
      $sagepay_pwd = get_option('eme_sagepay_test_pwd');
   } else {
      $sagepay_pwd = get_option('eme_sagepay_live_pwd');
   }

   // crypt is passed as part of the request
   $crypt=eme_sanitize_request($_GET['crypt']);
   require_once 'payment_gateways/sagepay/eme-sagepay-util.php';
   $decrypt = SagepayUtil::decryptAes($crypt, $sagepay_pwd);
   $decryptArr = SagepayUtil::queryStringToArray($decrypt);
   if ($decrypt && !empty($decryptArr)) {
      if ($decryptArr['Status']=='OK') {
         $payment_id=$decryptArr['VendorTxCode'];
         eme_update_payment_paid($payment_id,1,"sagepay",$decryptArr['VPSTxId']);
      }
   }
}

function eme_event_can_pay_online ($event) {
   if ($event['use_paypal'] || $event['use_2co'] || $event['use_webmoney'] || $event['use_fdgg'] || $event['use_mollie'] || $event['use_sagepay'] || $event['event_properties']['use_worldpay'] || $event['event_properties']['use_stripe'] || $event['event_properties']['use_braintree'] || $event['event_properties']['use_legacypaypal'] || $event['event_properties']['use_offline'])
      return 1;
   else
      return 0;
}
function eme_membership_can_pay_online ($membership) {
   if ($membership['properties']['use_paypal'] || $membership['properties']['use_2co'] || $membership['properties']['use_webmoney'] || $membership['properties']['use_fdgg'] || $membership['properties']['use_mollie'] || $membership['properties']['use_sagepay'] || $membership['properties']['use_worldpay'] || $membership['properties']['use_stripe'] || $membership['properties']['use_braintree'] || $membership['properties']['use_legacypaypal'] || $membership['properties']['use_offline'])
      return 1;
   else
      return 0;
}

function eme_create_member_payment($member_id) {
   global $wpdb;
   $payments_table = $wpdb->prefix.PAYMENTS_TBNAME;
   $members_table = $wpdb->prefix.MEMBERS_TBNAME;
   $payment_id = false;
   $payment=array();
   $payment['random_id']=eme_random_id();
   $payment['member_id']=$member_id;
   if ($wpdb->insert($payments_table,$payment)) {
      $payment_id = $wpdb->insert_id;
      $where['member_id'] = $member_id;
      $fields['transfer_nbr_be97'] = eme_transfer_nbr_be97($payment_id);
      $fields['payment_id'] = $payment_id;
      $wpdb->update($members_table, $fields, $where);
   }
   return $payment_id;
}

function eme_create_payment($booking_ids) {
   global $wpdb;
   $payments_table = $wpdb->prefix.PAYMENTS_TBNAME;
   $bookings_table = $wpdb->prefix.BOOKINGS_TBNAME;

   // some safety
   if (!$booking_ids)
      return false;

   $payment_id = false;
   $payment=array();
   $payment['random_id']=eme_random_id();
   if ($wpdb->insert($payments_table,$payment)) {
      $payment_id = $wpdb->insert_id;
      $booking_ids_arr=explode(",",$booking_ids);
      foreach ($booking_ids_arr as $booking_id) {
         $where = array();
         $fields = array();
         $where['booking_id'] = $booking_id;
         $fields['transfer_nbr_be97'] = eme_transfer_nbr_be97($payment_id);
         $fields['payment_id'] = $payment_id;
         $wpdb->update($bookings_table, $fields, $where);
      }
   }
   return $payment_id;
}

function eme_get_payment($payment_id,$payment_randomid=0) {
   global $wpdb;
   $payments_table = $wpdb->prefix.PAYMENTS_TBNAME;
   if ($payment_id)
      $sql = $wpdb->prepare("SELECT * FROM $payments_table WHERE id=%d",$payment_id);
   else
      $sql = $wpdb->prepare("SELECT * FROM $payments_table WHERE random_id=%s",$payment_randomid);
   $payment = $wpdb->get_row($sql, ARRAY_A);
   if (is_serialized($payment['pg_pids'])) {
              $tmp_pg_pids=unserialize($payment['pg_pids']);
	      foreach ($tmp_pg_pids as $pg_pid) {
		      $payment['pg_pids']=$pg_pid;
	      }
   }
   return $payment;

}

function eme_get_payment_booking_ids($payment_id) {
   global $wpdb;
   $table_name = $wpdb->prefix.BOOKINGS_TBNAME;
   $sql = $wpdb->prepare("SELECT booking_id FROM $table_name WHERE status=%d AND payment_id=%d",RSVP_STATUS_ACTIVE,$payment_id);
   return $wpdb->get_col($sql);
}

function eme_get_randompayment_booking_ids($payment_randomid) {
   global $wpdb;
   $payments_table = $wpdb->prefix.PAYMENTS_TBNAME;
   $bookings_table = $wpdb->prefix.BOOKINGS_TBNAME;
   $sql = $wpdb->prepare("SELECT bookings.booking_id FROM $bookings_table as bookings LEFT JOIN $payments_table as payments ON bookings.payment_id=payments.id where bookings.status=%d AND payments.random_id=%s",RSVP_STATUS_ACTIVE,$payment_randomid);
   return $wpdb->get_col($sql);
}

function eme_delete_payment($payment_id) {
   global $wpdb;
   $payments_table = $wpdb->prefix.PAYMENTS_TBNAME;
   $sql = $wpdb->prepare("DELETE FROM $payments_table WHERE id=%d",$payment_id);
   return $wpdb->get_var($sql);
}

function eme_get_payment_paid($payment) {
   $seats=0;
   if ($payment['member_id']>0) {
	   $member = eme_get_member($payment['member_id']);
	   return $member['paid'];
   } else {
	   $unpaid_count = eme_payment_count_unpaid_bookings($payment['id']);
	   if ($unpaid_count>0) return 0;
	   else return 1;
   }
}
function eme_get_payment_seats($payment) {
   $seats=0;
   // doesn't work for members of course
   if ($payment['member_id']>0) {
	   return 0;
   }

   $bookings = eme_get_bookings_by_paymentid($payment['id']);
   foreach ($bookings as $booking) {
      $seats += eme_get_total($booking['booking_seats']);
   }
   return $seats;
}

function eme_get_payment_price($payment_id) {
   $price=0;
   $bookings = eme_get_bookings_by_paymentid($payment_id);
   foreach ($bookings as $booking) {
      $price += eme_get_total_booking_price($booking);
   }
   return $price;
}

function eme_get_member_payment_price($payment_id) {
   $price=0;
   $member = eme_get_member_by_paymentid($payment_id);
   $price = eme_get_total_member_price($member);
   return $price;
}

function eme_update_attendance_count($payment_id) {
   global $wpdb;
   $table = $wpdb->prefix.PAYMENTS_TBNAME;
   $sql = "UPDATE $table SET attend_count=attend_count+1 WHERE id=$payment_id";
   $wpdb->query($sql);
}
function eme_get_attendance_count($payment_id) {
   global $wpdb;
   $table = $wpdb->prefix.PAYMENTS_TBNAME;
   $sql = "SELECT attend_count FROM $table WHERE id=$payment_id";
   return $wpdb->get_var($sql);
}

function eme_update_payment_pg_pids($payment_id, $pg_pid="") {
   global $wpdb;
   $table = $wpdb->prefix.PAYMENTS_TBNAME;
   $sql = $wpdb->prepare("UPDATE $table SET pg_pids=%s WHERE id=%d",$pg_pid,$payment_id);
   $wpdb->query($sql);
}

function eme_update_payment_paid($payment_id, $is_ipn=1, $pg="", $pg_pid="") {
   $payment = eme_get_payment($payment_id);
   if (isset($payment['member_id']) && $payment['member_id']>0) {
      $member_id=$payment['member_id'];
      $member=eme_get_member($member_id);
      $res=0;
      switch ($member['status']) {
       case MEMBER_STATUS_EXPIRED: $res=eme_renew_expired_member($member,$pg, $pg_pid);$action='extendMember';break;
       case MEMBER_STATUS_PENDING: $res=eme_member_set_paid($member,$pg, $pg_pid);$action='markPaid';break;
       case MEMBER_STATUS_ACTIVE: $res=eme_extend_member($member,$pg, $pg_pid);$action='extendMember';break;
       case MEMBER_STATUS_GRACE: $res=eme_extend_member($member,$pg, $pg_pid);$action='extendMember';break;
      }

      if ($res) {
	  $member = eme_get_member($member_id);
	  eme_email_member_action($member,$action);
      }
      if ($is_ipn) {
          eme_email_member_action($member,'ipnReceived');
      }
   } else {
      $booking_ids=eme_get_payment_booking_ids($payment_id);
      $total_price = eme_get_payment_price($payment_id);
      foreach ($booking_ids as $booking_id) {
        $booking=eme_get_booking($booking_id);
        $event = eme_get_event($booking['event_id']);
        if ($event['event_properties']['auto_approve'] && !$booking['booking_approved'] && !$booking['waitinglist']) {
           $res = eme_set_booking_paid_approved($booking_id, $pg, $pg_pid);
	   if ($res) {
		   //booking changed, let's get it again
		   $booking = eme_get_booking ($booking_id);
		   // if the option to send a mail after payment is received is active, we don't send a second mail for approval
		   // However: if the price to pay is 0, then no payment mail is sent ... so then we do send the approval mail
		   if (!get_option('eme_rsvp_mail_notify_paid') || $total_price==0)
			   eme_email_booking_action($booking,"approveBooking");
	   }
        } else {
           $res = eme_set_booking_paid($booking_id, $pg, $pg_pid);
	   if ($res) {
		   //booking changed, let's get it again
		   $booking = eme_get_booking ($booking_id);
	   }
        }
	if ($total_price>0) {
		eme_email_booking_action($booking,"paidBooking");
	}

        if ($is_ipn) {
		eme_email_booking_action($booking,'ipnReceived');
		if (has_action('eme_ipn_action')) do_action('eme_ipn_action',$booking);
        }
      }
   }
}

function eme_replace_payment_gateway_placeholders($format, $charge, $currency, $lang) {
   preg_match_all("/#_?[A-Za-z0-9_]+/", $format, $placeholders);

   usort($placeholders[0],'eme_sort_stringlenth');
   foreach($placeholders[0] as $result) {
      $replacement='';
      $found = 1;
      $orig_result = $result;
      if (preg_match('/#_EXTRACHARGE$/', $result)) {
         $replacement = $charge;
      } elseif (preg_match('/#_CURRENCY$/', $result)) {
         $replacement = $currency;
      } elseif (preg_match('/#_CURRENCYSYMBOL$/', $result)) {
         $replacement = eme_localized_currencysymbol($currency);
      } else {
         $found = 0;
      }
      if ($found)
         $format = str_replace($orig_result, $replacement ,$format );
   }

   // now, replace any language tags found in the format itself
   $format = eme_translate($format,$lang);

   return do_shortcode($format);
}

function eme_payment_count_unpaid_bookings($payment_id) {
   global $wpdb;
   $table_name = $wpdb->prefix.BOOKINGS_TBNAME;
   $sql = $wpdb->prepare("SELECT COUNT(*) FROM $table_name WHERE status=%d AND payment_id=%d AND booking_paid=0",RSVP_STATUS_ACTIVE,$payment_id);
   return $wpdb->get_var($sql);
}

?>
